/*

noise_source.c

Variable Noise (mono/stereo)

(c)2005 Artemiy Pavlov

*/

/* Includes: */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "ladspa.h"

/* The port numbers for the plugin: */

#define NOISE_TYPE 0
#define NOISE_DENSITY 1
#define NOISE_OUTPUT1 2
#define NOISE_OUTPUT2 3

/* The structure used to hold port connection information and state  */

typedef struct {

	LADSPA_Data * m_pfType;
	LADSPA_Data * m_pfDensity;
	LADSPA_Data * m_pfOutputBuffer1;
	LADSPA_Data * m_pfOutputBuffer2;
  
	int m_fStep;
	LADSPA_Data m_fNoise1;
	LADSPA_Data m_fNoise2;

} Noise;

/* Construct a new plugin instance. */

LADSPA_Handle instantiateNoise(const LADSPA_Descriptor * Descriptor, unsigned long SampleRate) {

	return malloc(sizeof(Noise));
	
}

/* Connect a port to a data location. */

void connectPortToNoise(LADSPA_Handle Instance, unsigned long Port, LADSPA_Data * DataLocation) {

	Noise * psNoise;

	psNoise = (Noise *)Instance;
	
	switch (Port) {
		case NOISE_TYPE:
			psNoise->m_pfType = DataLocation;
    		break;
		case NOISE_DENSITY:
			psNoise->m_pfDensity = DataLocation;
    		break;
  		case NOISE_OUTPUT1:
    		psNoise->m_pfOutputBuffer1 = DataLocation;
    		break;
  		case NOISE_OUTPUT2:
		    psNoise->m_pfOutputBuffer2 = DataLocation;
    		break;
  }
  
}

/* Noise mono */

void runMonoNoise(LADSPA_Handle Instance, unsigned long SampleCount) {
  
	LADSPA_Data * pfOutput;
	LADSPA_Data fType;
	LADSPA_Data fDensity;
	
	LADSPA_Data fNoise = 0;
	int fStep = 0;
	
	Noise * psNoise;
	unsigned long lSampleIndex;

	psNoise = (Noise *)Instance;
	
	fType = *(psNoise->m_pfType);
	fDensity = *(psNoise->m_pfDensity);
	
	if(fType >= 1.0f && fType < 2.0f){
	
		fType = 1;
		
		fDensity= 100.0f * (1 - fDensity);
		
	}
	
	if(fType >= 2.0f && fType < 3.0f){
	
		fType = 2;
		
		fDensity = 1.0f * pow ((1 - fDensity), 0.1f);
		
	}
	
	if(fType == 0){
	
		fStep = psNoise->m_fStep;
		fNoise = psNoise->m_fNoise1;
		
		if(fNoise == 0){
			fNoise = 2.0f * random()/RAND_MAX - 1.0f;
		}
		
	}
	
	pfOutput = psNoise->m_pfOutputBuffer1;
	
	for(lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++){	
	
		if(fType == 1){
	
			fStep = fStep + 1;
			
			if(fStep >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
				fStep = 0;
			}
			
		}
		
		if(fType == 2){
	
			fNoise = 1.0f * random()/RAND_MAX;
			
			if(fNoise >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
			} else {
				fNoise = 0.0f;
			}
			
		}
		
		*(pfOutput++) = fNoise;
			
	}
	
	if(fType == 1){
		psNoise->m_fStep = fStep;
		psNoise->m_fNoise1 = fNoise;
	}
	
}

/* Noise stereo */

void runStereoNoise(LADSPA_Handle Instance, unsigned long SampleCount) {
  
	LADSPA_Data * pfOutput;
	LADSPA_Data fType;
	LADSPA_Data fDensity;
	
	LADSPA_Data fNoise = 0;
	int fStep = 0;
	
	Noise * psNoise;
	unsigned long lSampleIndex;

	psNoise = (Noise *)Instance;
	
	fType = *(psNoise->m_pfType);
	fDensity = *(psNoise->m_pfDensity);
	
	if(fType >= 1.0f && fType < 2.0f){
	
		fType = 1;
		
		fDensity= 100.0f * (1 - fDensity);
		
	}
	
	if(fType >= 2.0f && fType < 3.0f){
	
		fType = 2;
		
		fDensity = 1.0f * pow ((1 - fDensity), 0.1f);
		
	}
	
	if(fType == 0){
	
		fStep = psNoise->m_fStep;
		fNoise = psNoise->m_fNoise1;
		
		if(fNoise == 0){
			fNoise = 2.0f * random()/RAND_MAX - 1.0f;
		}
		
	}
	
	pfOutput = psNoise->m_pfOutputBuffer1;
	
	for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++){
		
		if(fType == 1){
	
			fStep = fStep + 1;
			
			if(fStep >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
				fStep = 0;
			}
			
		}
		
		if(fType == 2){
	
			fNoise = 1.0f * random()/RAND_MAX;
			
			if(fNoise >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
			} else {
				fNoise = 0.0f;
			}
			
		}
					
		*(pfOutput++) = fNoise;
		
	}

	if(fType == 0){
	
		fStep = psNoise->m_fStep;
		fNoise = psNoise->m_fNoise2;
		
		if(fNoise == 0){
			fNoise = 2.0f * random()/RAND_MAX - 1.0f;
		}
		
	}
	
	pfOutput = psNoise->m_pfOutputBuffer2;
	
	for (lSampleIndex = 0; lSampleIndex < SampleCount; lSampleIndex++){
	
		if(fType == 1){
	
			fStep = fStep + 1;
			
			if(fStep >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
				fStep = 0;
			}
			
		}
		
		if(fType == 2){
	
			fNoise = 1.0f * random()/RAND_MAX;
			
			if(fNoise >= fDensity){
				fNoise = 2.0f * random()/RAND_MAX - 1.0f;
			} else {
				fNoise = 0.0f;
			}
			
		}
				
		*(pfOutput++) = fNoise;
		
	}
	
	if(fType == 1){
		psNoise->m_fStep = fStep;
		psNoise->m_fNoise2 = fNoise;
	}
}

/* Noise cleanup */

void cleanupNoise(LADSPA_Handle Instance) {

	Noise * psNoise;
	psNoise = (Noise *)Instance;

	free(psNoise);
}

LADSPA_Descriptor * g_psMonoDescriptor = NULL;
LADSPA_Descriptor * g_psStereoDescriptor = NULL;

/* Noise _init() function */

void _init() {

  char ** pcPortNames;
  LADSPA_PortDescriptor * piPortDescriptors;
  LADSPA_PortRangeHint * psPortRangeHints;

  g_psMonoDescriptor
    = (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));
  g_psStereoDescriptor 
    = (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));

  if (g_psMonoDescriptor) {
  
    g_psMonoDescriptor->UniqueID
      = 2541;
    g_psMonoDescriptor->Label
      = strdup("VariNoiseM");
    g_psMonoDescriptor->Properties
      = LADSPA_PROPERTY_HARD_RT_CAPABLE;
    g_psMonoDescriptor->Name 
      = strdup("Variable Noise (mono)");
    g_psMonoDescriptor->Maker
      = strdup("Artemiy Pavlov");
    g_psMonoDescriptor->Copyright
      = strdup("(c)2005 GPL");
    g_psMonoDescriptor->PortCount
      = 3;
    piPortDescriptors
      = (LADSPA_PortDescriptor *)calloc(3, sizeof(LADSPA_PortDescriptor));
    g_psMonoDescriptor->PortDescriptors
      = (const LADSPA_PortDescriptor *)piPortDescriptors;
	  
	piPortDescriptors[NOISE_TYPE]
	= LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
	piPortDescriptors[NOISE_DENSITY]
	= LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
	piPortDescriptors[NOISE_OUTPUT1]
	= LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
	  
    pcPortNames
      = (char **)calloc(3, sizeof(char *));
	g_psMonoDescriptor->PortNames 
      = (const char **)pcPortNames;
	  
	pcPortNames[NOISE_TYPE]
	= strdup("Noise Type");
	pcPortNames[NOISE_DENSITY]
	= strdup("Density");
	pcPortNames[NOISE_OUTPUT1]
	= strdup("Output");
	  
    psPortRangeHints = ((LADSPA_PortRangeHint *)
		calloc(3, sizeof(LADSPA_PortRangeHint)));
    g_psMonoDescriptor->PortRangeHints
      = (const LADSPA_PortRangeHint *)psPortRangeHints;
	 
	psPortRangeHints[NOISE_TYPE].HintDescriptor
	= (LADSPA_HINT_BOUNDED_BELOW 
	| LADSPA_HINT_BOUNDED_ABOVE
	| LADSPA_HINT_INTEGER
	| LADSPA_HINT_DEFAULT_1);
	psPortRangeHints[NOISE_TYPE].LowerBound
	= 1;
	psPortRangeHints[NOISE_TYPE].UpperBound
	= 2;
	  
	psPortRangeHints[NOISE_DENSITY].HintDescriptor
	= (LADSPA_HINT_BOUNDED_BELOW 
	| LADSPA_HINT_BOUNDED_ABOVE
	| LADSPA_HINT_DEFAULT_1);
	psPortRangeHints[NOISE_DENSITY].LowerBound
	= 0;
	psPortRangeHints[NOISE_DENSITY].UpperBound
	= 1;
	 
    psPortRangeHints[NOISE_OUTPUT1].HintDescriptor
	 = 0;
	  
    g_psMonoDescriptor->instantiate 
      = instantiateNoise;
    g_psMonoDescriptor->connect_port 
      = connectPortToNoise;
    g_psMonoDescriptor->activate
      = NULL;
    g_psMonoDescriptor->run
      = runMonoNoise;
    g_psMonoDescriptor->run_adding
      = NULL;
    g_psMonoDescriptor->set_run_adding_gain
      = NULL;
    g_psMonoDescriptor->deactivate
      = NULL;
    g_psMonoDescriptor->cleanup
      = cleanupNoise;
  }
  
  if (g_psStereoDescriptor) {
    
    g_psStereoDescriptor->UniqueID
      = 2542;
    g_psStereoDescriptor->Label
      = strdup("VariNoiseS");
    g_psStereoDescriptor->Properties
      = LADSPA_PROPERTY_HARD_RT_CAPABLE;
    g_psStereoDescriptor->Name 
      = strdup("Variable Noise (stereo)");
    g_psStereoDescriptor->Maker
      = strdup("Artemiy Pavlov");
    g_psStereoDescriptor->Copyright
      = strdup("(c)2005 GPL");
    g_psStereoDescriptor->PortCount
      = 4;
    piPortDescriptors
      = (LADSPA_PortDescriptor *)calloc(4, sizeof(LADSPA_PortDescriptor));
    g_psStereoDescriptor->PortDescriptors
      = (const LADSPA_PortDescriptor *)piPortDescriptors;
	  
	piPortDescriptors[NOISE_TYPE]
	= LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
	piPortDescriptors[NOISE_DENSITY]
	= LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
	piPortDescriptors[NOISE_OUTPUT1]
	= LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
	piPortDescriptors[NOISE_OUTPUT2]
	= LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
	  
    pcPortNames
      = (char **)calloc(4, sizeof(char *));
    g_psStereoDescriptor->PortNames 
      = (const char **)pcPortNames;
	  
    pcPortNames[NOISE_TYPE]
	= strdup("Noise Type");
	pcPortNames[NOISE_DENSITY]
	= strdup("Density");
	pcPortNames[NOISE_OUTPUT1]
	= strdup("Output L");
	pcPortNames[NOISE_OUTPUT2]
	= strdup("Output R");
	  
    psPortRangeHints = ((LADSPA_PortRangeHint *)
		calloc(4, sizeof(LADSPA_PortRangeHint)));
    g_psStereoDescriptor->PortRangeHints
      = (const LADSPA_PortRangeHint *)psPortRangeHints;
	  
	psPortRangeHints[NOISE_TYPE].HintDescriptor
	= (LADSPA_HINT_BOUNDED_BELOW 
	| LADSPA_HINT_BOUNDED_ABOVE
	| LADSPA_HINT_INTEGER
	| LADSPA_HINT_DEFAULT_1);
	psPortRangeHints[NOISE_TYPE].LowerBound
	= 1;
	psPortRangeHints[NOISE_TYPE].UpperBound
	= 2;
	  
	psPortRangeHints[NOISE_DENSITY].HintDescriptor
	= (LADSPA_HINT_BOUNDED_BELOW 
	| LADSPA_HINT_BOUNDED_ABOVE
	| LADSPA_HINT_DEFAULT_1);
	psPortRangeHints[NOISE_DENSITY].LowerBound
	= 0;
	psPortRangeHints[NOISE_DENSITY].UpperBound
	= 1;
	  
	psPortRangeHints[NOISE_OUTPUT1].HintDescriptor
	= 0;
	psPortRangeHints[NOISE_OUTPUT2].HintDescriptor
	= 0;
	  
    g_psStereoDescriptor->instantiate 
      = instantiateNoise;
    g_psStereoDescriptor->connect_port 
      = connectPortToNoise;
    g_psStereoDescriptor->activate
      = NULL;
    g_psStereoDescriptor->run
      = runStereoNoise;
    g_psStereoDescriptor->run_adding
      = NULL;
    g_psStereoDescriptor->set_run_adding_gain
      = NULL;
    g_psStereoDescriptor->deactivate
      = NULL;
    g_psStereoDescriptor->cleanup
      = cleanupNoise;
  }
}

void deleteDescriptor(LADSPA_Descriptor * psDescriptor) {
  unsigned long lIndex;
  if (psDescriptor) {
    free((char *)psDescriptor->Label);
    free((char *)psDescriptor->Name);
    free((char *)psDescriptor->Maker);
    free((char *)psDescriptor->Copyright);
    free((LADSPA_PortDescriptor *)psDescriptor->PortDescriptors);
    for (lIndex = 0; lIndex < psDescriptor->PortCount; lIndex++)
      free((char *)(psDescriptor->PortNames[lIndex]));
    free((char **)psDescriptor->PortNames);
    free((LADSPA_PortRangeHint *)psDescriptor->PortRangeHints);
    free(psDescriptor);
  }
}

void _fini() {
  deleteDescriptor(g_psMonoDescriptor);
  deleteDescriptor(g_psStereoDescriptor);
}

const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index) {
  switch (Index) {
  case 0:
    return g_psMonoDescriptor;
  case 1:
    return g_psStereoDescriptor;
  default:
    return NULL;
  }
}
